==> dialserver-0.3.1.tar.gz: dialserver-0.3.1/conf.c <==
// conf.c
//
// Configuration file acquisition
//
// Copyright (c) 2001,2002 Andrew McGill and Leading Edge Business Solutions
//
// This file is part of dialserver.
//
// dialserver is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// dialserver is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// dialserver; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA

#include "common.h"
#include "util.h"
#include "shell.h"
#include "conf.h"

char *config_file_image;

// Read a stringy thingy from the configuration file
char *cfg_readstr(char *prompt, char *adefault)
{
	char *p;
	int promptlen = strlen(prompt);
	char *result;

	for (p=config_file_image; *p; p=p+strlen(p)+1) {
		if (strncasecmp(p,prompt,promptlen)==0 && p[promptlen]=='=') {
			result = xstrdup(p+promptlen+1);
			return result;
		}
	}
	// not found -- allocate and return default
	result = xstrdup(adefault);
	return result; 
}

// Read an integer inta tha memory from the config file
int cfg_readint(char *prompt, int adefault)
{
	char *stringvalue;
	int value;

	value = adefault;
	stringvalue = cfg_readstr(prompt,NULL);
	if (stringvalue) {
		value = atoi(stringvalue);
		free(stringvalue);
	}
	return value;
}

// Read a boolean into memory from the config file.
int cfg_readbool(char *prompt, int adefault)
{
	char *stringvalue;
	int value;

	value = adefault;
	stringvalue = cfg_readstr(prompt,NULL);
	if (stringvalue) {
		switch (stringvalue[0]) {
		case '1':
		case 'e':  // e nabled
		case 'E':  
		case 'y':  // y es
		case 'Y':
			value=1;
			break;
		case '0':
		case 'd':  // d isabled
		case 'D':  
		case 'n':  // y es
		case 'N':
			value=0;
			break;
		default:
			value = -1;
			break;
		}
		free(stringvalue);
	}
	return value;
}


static char *config_filename;
static char *config_argv0;

// Load the defaults from a config file.  If the filename
// provided is NULL, reload the config file supplied
int dlr_reloadconfig(void)
{
	int size=24000;
	char command[128];

	if (config_file_image) free(config_file_image);
	config_file_image=xmalloc(size);
	command[sizeof(command)-1] = 0;
	// Read the configuration from the output of /bin/sh ...
	//   set -a
	//   . /etc/dialserver.conf
	//   dialserver showenv
	snprintf(command,sizeof(command)-1,"set -a;. %s;%s showenv",
		config_filename, config_argv0);
	shell_inline(command,config_file_image,&size);
	// Set defaults, and read actual values from the config file

	// Macros for loading and reloading the configuration file
#define GETCONF_STR(x,dflt) \
	if (conf.x) free(conf.x) ; \
	conf.x = cfg_readstr( #x, dflt ); \
	// fprintf(stderr,"%s: %s\n",#x,conf.x); 
	
#define GETCONF_INT(x,dflt) \
	conf.x = cfg_readint(#x, dflt); 

#define GETCONF_INT_RANGE(x,dflt,min,max)			\
	GETCONF_INT(x,dflt); 					\
	if (conf.x < min || conf.x > max) {			\
		fprintf(stderr,"Configuration value for %s "	\
			"must be between %d and %d\n",		\
			#x, min,max);				\
		exit(1);					\
	}

#define GETCONF_BOOL(x,dflt)					\
	conf.x = cfg_readbool(#x, dflt); 			\
	if (conf.x < 0) {					\
		fprintf(stderr,"Configuration value for %s "	\
			"must be `yes' or `no'\n",	\
			#x);					\
		exit(1);					\
	}

	// See the comments in the standard dialserver.conf for 
	// an explanation of the fields ...
	GETCONF_STR(cmdcheck,  		"/sbin/ifconfig ppp0");
	GETCONF_STR(cmddiagnose,  	
		"echo [[Last dial]];"
		"cat /var/log/wvdial.log;"
		"echo [[Network interfaces]];"
		"/sbin/ifconfig;"
		"echo [[Network routes]];"
		"/sbin/route -n;"
		"echo [[PPP messages]];"
		"tail -200 /var/log/messages|egrep -i 'ppp|isdn'|tail -20;"
		"echo [[DNS configuration]];"
		"egrep -v '^(#|$)' /etc/resolv.conf"
		);
	GETCONF_STR(reg_up, 		"inet");
	GETCONF_STR(reg_notup, 		"192\\.168");
	GETCONF_STR(cmdpackets,		"/sbin/ifconfig ppp0");
	GETCONF_STR(reg_rx, 		"RX[^0-9]+([0-9]+)");
	GETCONF_STR(reg_tx, 		"TX[^0-9]+([0-9]+)");
	GETCONF_STR(reg_linkisvalid, 	"^"
		"[0-2]?[0-9]?[0-9]\\."
		"[0-2]?[0-9]?[0-9]\\."
		"[0-2]?[0-9]?[0-9]\\."
		"[0-2]?[0-9]?[0-9]$");
	GETCONF_STR(reg_islocal, 
		// http:// or ftp:// or anything
		"[a-z]+://"
		// optional user:password@
		"(|[^@/]*@)"
		// Unqualified alpha only or private ip address range - 
		// this is C code, so the "\\" actually means just '\'
		"(gabriel.+|[a-z]+|(10|192\\.168|172\\.1[678])\\.[0-9.]*)"
		// Optional port number
		"(|:[0-9]+)/" ) ;
	GETCONF_STR(reg_simplifyurl,
		// http:// or ftp:// or anything
		"[a-z]+://"
		// optional user:password@
		"(|[^@/]*@)"
		// Anything except the trailing slash and port number
		"([^/:]+)") ;
	GETCONF_INT(timeoutdial,	40);
	GETCONF_INT(timeouthangup,	15);
	GETCONF_INT(timeminimumcall,	90);	// Don't hang up for the first 90 seconds of a new user
	GETCONF_INT(timepacketcheck,	120);	// Time over which to measure packets
	GETCONF_INT(minimumpackets,	20);	// Default: 20 packets in 2 minutes
	GETCONF_INT(minimumpacketspermin,0);	// Use minimumpackets
	if (conf.minimumpacketspermin) {
		conf.minimumpackets = conf.minimumpacketspermin *
			conf.timepacketcheck / 60;
	}
	GETCONF_STR(unixsocket,		"/var/run/dialctl");
	GETCONF_STR(pidfile,  		"/var/run/dialserver.pid");
	GETCONF_INT(tcpport,		2200);
	GETCONF_INT_RANGE(timepoll, 	5, 1, 60);
	GETCONF_STR(passwdfile,		"/etc/dialpasswd");
	GETCONF_STR(cmdcheckpasswd,	"");
	
	// Dialling
	GETCONF_STR(cmdbeforedial,	"");
	GETCONF_STR(cmddial,		"{date;wvdial;} >& /var/log/wvdial.out");
	GETCONF_STR(cmdafterdial,	"");
	GETCONF_STR(cmddialtimeout,	"");
	// Waking up to send and recieve mail every n seconds
	GETCONF_STR(cmdrefresh,		"/usr/sbin/sendreceivemail");
	GETCONF_INT_RANGE(timerefresh, 	600, 0, 6000);
	// Command run while link is up
	GETCONF_STR(cmdduring,		"");
	// Hanging up
	GETCONF_STR(cmdbeforehangup,	"");
	GETCONF_STR(cmdhangup,		"killall wvdial");
	GETCONF_STR(cmdafterhangup,	"");
	GETCONF_STR(cmdhanguptimeout,	"killall pppd; sleep 1; killall -9 wvdial; killall -9 pppd");
	// Log out
	GETCONF_STR(cmdlogin,		"");
	GETCONF_STR(cmdiptoname,	"echo $dsUSERIP");
	GETCONF_STR(cmdlogout,		"");
	// Logging
	GETCONF_STR(logfile,		"/var/log/dialserver.log");
	// Authorisation
	GETCONF_BOOL(autoauth, 	1);

	free(config_file_image);
	config_file_image = NULL;
	return 0;
}

int dlr_loadconfig(char *afilename, char *anargv0)
{
	// Choose a file
	if (afilename) config_filename = xstrdup(afilename);
	// Choose a program to run
	if (anargv0) config_argv0 = xstrdup(anargv0);
	return dlr_reloadconfig();
}

==> dialserver-0.3.3.tar.gz: dialserver-0.3.3/conf.c <==
// conf.c
//
// Configuration file acquisition
//
// Copyright (c) 2001,2002 Andrew McGill and Leading Edge Business Solutions
//
// This file is part of dialserver.
//
// dialserver is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// dialserver is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// dialserver; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA

#include "common.h"
#include "util.h"
#include "shell.h"
#include "conf.h"

char *config_file_image;

// Read a stringy thingy from the configuration file
char *cfg_readstr(char *prompt, char *adefault)
{
	char *p;
	int promptlen = strlen(prompt);
	char *result;

	for (p=config_file_image; *p; p=p+strlen(p)+1) {
		if (strncasecmp(p,prompt,promptlen)==0 && p[promptlen]=='=') {
			result = xstrdup(p+promptlen+1);
			return result;
		}
	}
	// not found -- allocate and return default
	result = xstrdup(adefault);
	return result; 
}

// Read an integer inta tha memory from the config file
int cfg_readint(char *prompt, int adefault)
{
	char *stringvalue;
	int value;

	value = adefault;
	stringvalue = cfg_readstr(prompt,NULL);
	if (stringvalue) {
		value = atoi(stringvalue);
		free(stringvalue);
	}
	return value;
}

// Read a boolean into memory from the config file.
int cfg_readbool(char *prompt, int adefault)
{
	char *stringvalue;
	int value;

	value = adefault;
	stringvalue = cfg_readstr(prompt,NULL);
	if (stringvalue) {
		switch (stringvalue[0]) {
		case '1':
		case 'e':  // e nabled
		case 'E':  
		case 'y':  // y es
		case 'Y':
			value=1;
			break;
		case '0':
		case 'd':  // d isabled
		case 'D':  
		case 'n':  // y es
		case 'N':
			value=0;
			break;
		default:
			value = -1;
			break;
		}
		free(stringvalue);
	}
	return value;
}


static char *config_filename;
static char *config_argv0;

// Load the defaults from a config file.  If the filename
// provided is NULL, reload the config file supplied
int dlr_reloadconfig(void)
{
	int size=24000;
	char command[128];

	if (config_file_image) free(config_file_image);
	config_file_image=xmalloc(size);
	command[sizeof(command)-1] = 0;
	// Read the configuration from the output of /bin/sh ...
	//   set -a
	//   . /etc/dialserver.conf
	//   dialserver showenv
	snprintf(command,sizeof(command)-1,"set -a;. %s;%s showenv",
		config_filename, config_argv0);
	shell_inline(command,config_file_image,&size);
	// Set defaults, and read actual values from the config file

	// Macros for loading and reloading the configuration file
#define GETCONF_STR(x,dflt) \
	if (conf.x) free(conf.x) ; \
	conf.x = cfg_readstr( #x, dflt ); \
	// fprintf(stderr,"%s: %s\n",#x,conf.x); 
	
#define GETCONF_INT(x,dflt) \
	conf.x = cfg_readint(#x, dflt); 

#define GETCONF_INT_RANGE(x,dflt,min,max)			\
	GETCONF_INT(x,dflt); 					\
	if (conf.x < min || conf.x > max) {			\
		fprintf(stderr,"Configuration value for %s "	\
			"must be between %d and %d\n",		\
			#x, min,max);				\
		exit(1);					\
	}

#define GETCONF_BOOL(x,dflt)					\
	conf.x = cfg_readbool(#x, dflt); 			\
	if (conf.x < 0) {					\
		fprintf(stderr,"Configuration value for %s "	\
			"must be `yes' or `no'\n",	\
			#x);					\
		exit(1);					\
	}

	// See the comments in the standard dialserver.conf for 
	// an explanation of the fields ...
	GETCONF_STR(cmdcheck,  		"/sbin/ifconfig ppp0");
	GETCONF_STR(cmddiagnose,  	
		"echo [[Last dial]];"
		"cat /var/log/wvdial.log;"
		"echo [[Network interfaces]];"
		"/sbin/ifconfig;"
		"echo [[Network routes]];"
		"/sbin/route -n;"
		"echo [[PPP messages]];"
		"tail -200 /var/log/messages|egrep -i 'ppp|isdn'|tail -20;"
		"echo [[DNS configuration]];"
		"egrep -v '^(#|$)' /etc/resolv.conf"
		);
	GETCONF_STR(reg_up, 		"inet");
	GETCONF_STR(reg_notup, 		"192\\.168");
	GETCONF_STR(cmdpackets,		"/sbin/ifconfig ppp0");
	GETCONF_STR(reg_rx, 		"RX[^0-9]+([0-9]+)");
	GETCONF_STR(reg_tx, 		"TX[^0-9]+([0-9]+)");
	GETCONF_STR(reg_linkisvalid, 	"^"
		"[0-2]?[0-9]?[0-9]\\."
		"[0-2]?[0-9]?[0-9]\\."
		"[0-2]?[0-9]?[0-9]\\."
		"[0-2]?[0-9]?[0-9]$");
	GETCONF_STR(reg_islocal, 
		// http:// or ftp:// or anything
		"[a-z]+://"
		// optional user:password@
		"(|[^@/]*@)"
		// Unqualified alpha only or private ip address range - 
		// this is C code, so the "\\" actually means just '\'
		"(gabriel.+|[a-z]+|(10|192\\.168|172\\.1[678])\\.[0-9.]*)"
		// Optional port number
		"(|:[0-9]+)/" ) ;
	GETCONF_STR(reg_simplifyurl,
		// http:// or ftp:// or anything
		"[a-z]+://"
		// optional user:password@
		"(|[^@/]*@)"
		// Anything except the trailing slash and port number
		"([^/:]+)") ;
	GETCONF_INT(timeoutdial,	40);
	GETCONF_INT(timeouthangup,	15);
	GETCONF_INT(timeminimumcall,	90);	// Don't hang up for the first 90 seconds of a new user
	GETCONF_INT(timepacketcheck,	120);	// Time over which to measure packets
	GETCONF_INT(minimumpackets,	20);	// Default: 20 packets in 2 minutes
	GETCONF_INT(minimumpacketspermin,0);	// Use minimumpackets
	if (conf.minimumpacketspermin) {
		conf.minimumpackets = conf.minimumpacketspermin *
			conf.timepacketcheck / 60;
	}
	GETCONF_INT(timelinkidle,	(conf.timepacketcheck*3/2));	
	GETCONF_INT(timeidlecheck,	60);	
	GETCONF_STR(unixsocket,		"/var/run/dialctl");
	GETCONF_STR(pidfile,  		"/var/run/dialserver.pid");
	GETCONF_INT(tcpport,		2200);
	GETCONF_INT_RANGE(timepoll, 	5, 1, 60);
	GETCONF_STR(passwdfile,		"/etc/dialpasswd");
	GETCONF_STR(cmdcheckpasswd,	"");
	
	// Dialling
	GETCONF_STR(cmdbeforedial,	"");
	GETCONF_STR(cmddial,		"{date;wvdial;} >& /var/log/wvdial.out");
	GETCONF_STR(cmdafterdial,	"");
	GETCONF_STR(cmddialtimeout,	"");
	// Waking up to send and recieve mail every n seconds
	GETCONF_STR(cmdrefresh,		"/usr/local/bin/sendreceivemail");
	GETCONF_INT_RANGE(timerefresh, 	600, 0, 6000);
	GETCONF_STR(cmdwakeoff,		"/usr/local/bin/dialservercheck");
	GETCONF_INT_RANGE(timewakeoff, 	180, 0, 6000);
	// Command run while link is up
	GETCONF_STR(cmdduring,		"");
	// Detect idle links
	GETCONF_STR(cmdlistidle,	"");
	// Hanging up
	GETCONF_STR(cmdbeforehangup,	"");
	GETCONF_STR(cmdhangup,		"killall wvdial");
	GETCONF_STR(cmdafterhangup,	"");
	GETCONF_STR(cmdhanguptimeout,	"killall pppd; sleep 1; killall -9 wvdial; killall -9 pppd");
	// Log out
	GETCONF_STR(cmdlogin,		"");
	GETCONF_STR(cmdiptoname,	"echo $dsUSERIP");
	GETCONF_STR(cmdlogout,		"");
	// Logging
	GETCONF_STR(logfile,		"/var/log/dialserver.log");
	// Authorisation
	GETCONF_BOOL(autoauth, 	1);

	free(config_file_image);
	config_file_image = NULL;
	return 0;
}

int dlr_loadconfig(char *afilename, char *anargv0)
{
	// Choose a file
	if (afilename) config_filename = xstrdup(afilename);
	// Choose a program to run
	if (anargv0) config_argv0 = xstrdup(anargv0);
	return dlr_reloadconfig();
}
